// EE231002 Lab14. Image Processing
// 108061112, 林靖
// Date: Dec. 28, 2019

#include <stdio.h>
#include <stdlib.h>

typedef struct sPIXEL {
	unsigned char r, g, b;	//three color components
} PIXEL;

typedef struct sIMG {
	char header[3];	//header,either P3 or P6
	int W, H;	//width and height
	int level;	//intensity level of each color
	PIXEL **PX;	//2D array for all the pixels
} IMG;

IMG *PPMin(char *inFile);	//read pic
void PPMfout(IMG *p1, char *outFile);	//print pic
IMG *PPMcvt(IMG *p1, IMG *ee, IMG *nthu, int x1, int y1, int x2, int y2);	//compose pic

int main(int argc, char *argv[])
{
	IMG *back, *ee, *nthu;	//declare pointer points to pic

	back = PPMin(argv[1]);	//read background photo
	ee = PPMin(argv[2]);	//read EE logo
	nthu = PPMin(argv[3]);	//read NTHU logo
	PPMcvt(back, ee, nthu, 910, 1780, 1090, 1600);	//compose pic
	PPMfout(back, argv[4]);	//print pic

	return 0;	//program end
}

IMG* PPMin(char *inFile)
{
	int i, j;	//looping index
	IMG *pic;
	FILE *file;

	pic = (IMG *)malloc(sizeof(IMG));
	file = fopen(inFile, "r");	//open file
	fscanf(file, "%s", pic->header);	//scan P6
	fscanf(file, "%d %d\n%d\n", &pic->W, &pic->H, &pic->level);	//scan info
	pic->PX = (PIXEL **)malloc(pic->W * sizeof(PIXEL *));
	for (i = 0; i < pic->W; i++) {
		pic->PX[i] = (PIXEL*)malloc(pic->H * sizeof(PIXEL));
	}
	for (i = 0; i < pic->H; i++) {
		for (j = 0; j < pic->W; j++) {
			pic->PX[j][i].r = fgetc(file);	//get pixel
			pic->PX[j][i].g = fgetc(file);	//...
			pic->PX[j][i].b = fgetc(file);	//...
		}
	}
	fclose(file);	//close file

	return pic;	//return pointer
}

void PPMfout(IMG *p1, char *outFile)
{
	int i, j;
	FILE *fout;

	fout = fopen(outFile, "w");	//open foutput file
	fprintf(fout, "%s\n", p1->header);	//write header
	fprintf(fout, "%d %d\n", p1->W, p1->H);	//write width &height
	fprintf(fout, "%d\n", p1->level);	//write color level
	for (i = 0; i < p1->H; i++) {
		for (j = 0; j < p1->W; j++) {
			fprintf(fout, "%c", p1->PX[j][i].r);
			fprintf(fout, "%c", p1->PX[j][i].g);
			fprintf(fout, "%c", p1->PX[j][i].b);
		}
	}
	fclose(fout);	//close foutput file

	return;
}

IMG* PPMcvt(IMG *p1, IMG *ee, IMG *nthu, int x1, int y1, int x2, int y2)
{
	int i, j,
		gray,
		eeX, eeY,
		nthuX, nthuY;

	for (j = 0; j < p1->H; j++) {
		for (i = 0; i < p1->W; i++) {
			if (i < x1 ||
				i > x2 ||
				j < y2 ||
				j > y1) {
				gray = p1->PX[i][j].r * 0.2126 +
					   p1->PX[i][j].g * 0.7152 +
					   p1->PX[i][j].b * 0.0722;
				p1->PX[i][j].r = gray;
				p1->PX[i][j].g = gray;
				p1->PX[i][j].b = gray;
			}
		}
	}

	eeX = (p1->W) - (ee->W);	//find location for EE logo
	eeY = (p1->H) - (ee->H);	//...
	for (j = 0; j < ee->H; j++) {
		for (i = 0; i < ee->W; i++) {
			if (ee->PX[i][j].r != 255 ||
				ee->PX[i][j].g != 255 ||
				ee->PX[i][j].b != 255) {
				p1->PX[eeX + i][eeY + j] = ee->PX[i][j];
			}
		}
	}

	nthuX = (p1->W - nthu->W) / 2;	//find location for NTHU logo
	nthuY = (p1->H - nthu->H) / 2;	//...
	for (j = 0; j < nthu->H; j++) {
		for (i = 0; i < nthu->W; i++) {
			if (nthu->PX[i][j].r != 255 ||
				nthu->PX[i][j].g != 255 ||
				nthu->PX[i][j].b != 255) {
				p1->PX[nthuX + i][nthuY + j].r = 255;
				p1->PX[nthuX + i][nthuY + j].b = 255;
			}
		}
	}

	for (j = (y2 - 1); j <= (y1 + 1); j++) {
		for (i = (x1 - 1); i <= (x2 + 1); i++) {
			if (i <= (x1 + 1) ||
				i >= (x2 - 1) ||
				j <= (y2 + 1) ||
				j >= (y1 - 1)) {
				p1->PX[i][j].r = 0;
				p1->PX[i][j].g = 255;
				p1->PX[i][j].b = 255;
			}
		}
	}

	return p1;
}
